#include <QtCore/QCoreApplication>
#include <QStringList>
#include <QSettings>
#include <QObject>

#include "proximityserver.h"

/*
  Server lauscht auf 10.1.254.1:2342
 */

//! der Einstiegspunkt in das Programm
/*!
  Je nach Kompilierung startet diese Methode einen Server, oder einen Client
  \param argc Anzahl der Kommandozeilenparameter
  \param argv Vektor mit den Parametern
  \returns Statusint
  */
int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv); //!< die Applikation
    QCoreApplication::setOrganizationName("INKA");
    QCoreApplication::setApplicationName("ProximityServer");
    QSettings::setDefaultFormat(QSettings::IniFormat);
    QTextStream cout(stdout, QIODevice::WriteOnly);
    QStringList arguments(a.arguments());
    bool log = false;
    quint8 modus = NETWORK_PLAYER;
    QString sourceFile = "";
    if(arguments.contains("-l")|| arguments.contains("-log")) {
        log = true;
    }
    if (arguments.contains("-p") || arguments.contains("-play")) {
        int index = arguments.indexOf("-p");
        if(index == -1) {
            index = arguments.indexOf("-play");
        }
        if((index+1)<arguments.count()) {
            sourceFile = arguments.at(index+1);
            modus = LOGFILE_PLAYER;
        } else {
            QTextStream cerr(stderr, QIODevice::WriteOnly);
            cerr<<"Nach -p oder -play muss ein Logfile angegeben werden\n";
            exit(-3);
        }

    }else if(arguments.contains("-pl") || arguments.contains("-playloop")){
        int index = arguments.indexOf("-pl");
        if(index == -1) {
            index = arguments.indexOf("-playloop");
        }
        if((index+1)<arguments.count()) {
            sourceFile = arguments.at(index+1);
            modus = LOGFILE_PLAYER_LOOP;
        } else {
            QTextStream cerr(stderr, QIODevice::WriteOnly);
            cerr<<"Nach -pl oder -playloop muss ein Logfile angegeben werden\n";
            exit(-3);
        }
    } else if (arguments.contains("-s") || arguments.contains("-serial")) {
        int index = arguments.indexOf("-s");
        if(index == -1) {
            index = arguments.indexOf("-serial");
        }
        qDebug()<<arguments.count()<<" gegen "<<index;
        if((index+1)<arguments.count()) {
            sourceFile = arguments.at(index+1);
        }
        cout<<"Nutze einen seriellen Port.\n";
        cout.flush();
        modus = SERIAL_PLAYER;
    } // TODO: Multicast implementieren ...
    if(arguments.contains("-h") || arguments.contains("-help") || arguments.contains("--help") || arguments.contains("-?")) {
        cout<<"Hilfe zum ProximityServer\n";
        cout<<"Dieses Programm dient als Server zur OpenBeacon-Hardware\n\n";
        cout<<"Ausfuehrung der Applikation: \n";
        cout<<"   ProximityServer <Option> [<Datei>]\n\n";
        cout<<"Optionen: \n";
        cout<<"   -h     zeigt diese Hilfe an\n";
        cout<<"   -help  siehe -h\n";
        cout<<"   --help siehe -h\n";
        cout<<"   -?     siehe -h\n";
        cout<<"   ----------------------------------------------------------\n";
        cout<<"   -l     aktiviert das logging\n";
        cout<<"   -log   siehe -l\n";
        cout<<"   ----------------------------------------------------------\n";
        cout<<"   -p <Datei>         spielt eine log-datei ab\n";
        cout<<"   -play <Datei>      siehe -p\n";
        cout<<"   ----------------------------------------------------------\n";
        cout<<"   -pl <Datei>        spielt eine log-datei wiederholt ab\n";
        cout<<"   -playloop <Datei>  siehe -pl\n";
        cout<<"   ----------------------------------------------------------\n";
        cout<<"   -s <Schnittstelle> aktiviert das Empfangen der Nachrichten\n";
        cout<<"                      ueber serielle Konsole (OpenBeacon USB)\n";
        cout<<"   -serial <Schnittstelle> siehe -s\n"<<flush;
        exit(0);
    }
    ProximityServer m(modus, log, sourceFile);//!< der Server
    //QObject::connect(&m, SIGNAL(quitMe), &a, SLOT(quit()));
    if(m.start()) {
#ifdef _DEV_
        cout<<"gestartet!\n"<<flush;
#endif
    } else {
#ifdef _DEV_
        cout<<"aiaiai\n"<<flush;
#endif
    }
    signal(SIGTERM, SIG_DFL);
    return a.exec();
}
