#include "proximitydatareceiver.h"
#include "../common/protocol/openbeacon.h"

#include "attractiveforcelocator.h"

#include <QBuffer>

#ifdef Q_OS_WIN // hier muss dann aber noch zu Ws2_32.lib gelinkt werden  ... funktioniert ab Windows2000
#include <winsock2.h>
#else
#include <netinet/in.h>
#endif

#ifdef __MINGW32__
#include <stdint.h>
#define u_int8_t uint8_t
#define u_int16_t uint16_t
#define u_int32_t uint32_t
#endif


#include <QTextStream>
#include <QHostAddress>
#include <QSettings>
#include <QStringList>

#define MX ((z>>5)^(y<<2))+((y>>3)^(z<<4))^(sum^y)+(tea_key[p&3^e]^z)
#define DELTA 0x9E3779B9L

// dummy key
const long tea_key[4]={0xab94ec75, 0x160869c5, 0xfbf908da, 0x60bedc73};
#include "../common/protocol/proximitytag.h"

/*!
  konstruiert einen ProximityDataReceiver
  \param queue die Warteschlange, in die Events geschrieben werden sollen
  \param packageQueueMutex Der Mutex auf die Warteschlange
  \param logger Die Instanz des Loggers, wenn es einen gibt (ansonsten bitte 0 übergeben)
  \param model Das Datenmodell, auf dem alle Aktionen ausgeführt werden sollen
  */
ProximityDataReceiver::ProximityDataReceiver(QQueue<PTNP::Package*> &queue,
                                             QMutex &packageQueueMutex,
                                             ProximityLogger* logger,
                                             ProximityDataModel& model)
                                                 : _queue(queue),
                                                 _packageQueueMutex(packageQueueMutex),
                                                 _logger(logger),
                                                 _model(model)
{
#ifdef _DEV_
    QTextStream cout(stdout, QIODevice::WriteOnly);
    cout<<"pdr.cpp: init ProximityDataReceiver "<<flush;
#endif
    if(logger!=0) {
#ifdef _DEV_
        cout<<"with logger"<<flush;
        _log = true;
#endif
    }
    else _log = false;
#ifdef _DEV_
    cout<<"\n"<<flush;
#endif
}

/*!
  Tut momentan nichts, da keine privaten Variablen mit new erstellt werden
  */
ProximityDataReceiver::~ProximityDataReceiver() {
}

/*!
  Diese Methode verarbeitet empfangene Daten.
  Wenn geloggt werden soll, dann wird das hier ausgelöst.
  Es wird ausgewertet, ob die Länge und Prüfsumme stimmen,
  von welcher Antenne und welchem Tag die Daten gesendet wurden und
  ob die Antenne und das Tag bekannt sind. Schließlich wird
  das Paket anhand seiner Art analysiert und entsprechende
  Pakete für die Warteschlange generiert, bzw. \sa OuttimingObject s
  generiert.
  Am Ende wird das Ursprungsdatagram gelöscht!
  */
void ProximityDataReceiver::processDatagram(QByteArray* datagram, bool decrypt)
{
#ifdef _DEV_
    QTextStream cout(stdout, QIODevice::WriteOnly);
#endif

    // hier muss das Paket dekodiert werden

    if(datagram->count() == sizeof(TBeaconPackage) ) {
        if(_log) {
            _logger->log(*datagram);
        }
        if(decrypt) {
            // entschlüsseln des Datenpaketes
            QByteArray toBeDecrypted = datagram->mid(0,16); // erst Datenpaket holen
            decryptDatagram(&toBeDecrypted); // dann entschlüsseln
            datagram->replace(0,16,toBeDecrypted); // dann zurückschreiben
#ifdef _DEV_
            cout<<"pdr: decrypted: "<<flush;
            for(int i=0; i<datagram->count(); i++) {
                hex(cout)<<(quint8)datagram->at(i)<<" "<<flush;
            }
            cout<<"\n"<<flush;
#endif
        }

        QByteArray myMessage = datagram->mid(0,16); // die eigentliche Nachricht
        QByteArray antID = datagram->mid(16,4);
        // Wenn die CRC-Summe für das Paket stimmt
        if(CRC16((unsigned char*)(myMessage.data()), myMessage.size()) == 0) {
            PTNP::LocatableProximityTag* tag;
            PTNP::ProximityAntenna* antenna;
            PTNP::aid antennaID;
            quint32 a;
            PTNP::oid tagOid;
            PTNP::signalStrength strength;
            PTNP::sequenceNumber seq;
            TBeaconEnvelope* tbeacon;

            //antennaID = host->toIPv4Address()& 0x0000ffff; // nur die letzten zwei Bytes der Antennenadresse sind interessant
            a = (quint32)*antID.data();
            antennaID = a & 0x0000ffff;
            antenna = antennaReference(antennaID);
#ifdef _DEV_
            dec(cout)<<"pdr.cpp: antID: "<<antenna->aid()<<" bzw. "<<a<<" sendet "<<flush;
#endif

            // das ist das Paket, welches die Antennen senden
            tbeacon = (TBeaconEnvelope *) myMessage.data();

            bool unknown = false; // Flag für unbekannte Pakete

            if(tbeacon->pkt.proto != RFBPROTO_BEACONTRACKER &&
               tbeacon->pkt.proto != RFBPROTO_PROXREPORT &&
               tbeacon->pkt.proto != RFBPROTO_PROXTRACKER) {
                unknown = true;
            }
            else {
                // Umwandlung der allgemeinen Werte
                tbeacon->pkt.oid = ntohs(tbeacon->pkt.oid);
                tbeacon->pkt.crc = ntohs(tbeacon->pkt.crc);

                tagOid = tbeacon->pkt.oid;
                tag = tagReference(tagOid);

                switch(tbeacon->pkt.proto) {
                case RFBPROTO_BEACONTRACKER:
#ifdef _DEV_
                    cout<<"einen BeaconTracker von "<<tagOid<<"; Signalstaerke: "<<tbeacon->pkt.p.tracker.strength<<endl<<flush;
#endif
                    // hier Umwandlung aller relevanten Werte
                    tbeacon->pkt.p.tracker.oid_last_seen = ntohs(tbeacon->pkt.p.tracker.oid_last_seen);
                    tbeacon->pkt.p.tracker.powerup_count = ntohs(tbeacon->pkt.p.tracker.powerup_count);
                    tbeacon->pkt.p.tracker.seq = ntohl(tbeacon->pkt.p.tracker.seq);
                    strength = tbeacon->pkt.p.tracker.strength;
                    seq = (PTNP::sequenceNumber)tbeacon->pkt.p.tracker.seq>>3;
                    tag->seen(antennaID,strength);
                    createBeaconTrackerPackage(tagOid, strength, antennaID, seq);
                    break;
                case RFBPROTO_PROXREPORT:
                    tbeacon->pkt.p.prox.seq = ntohs(tbeacon->pkt.p.prox.seq);
#ifdef _DEV_
                    cout<<"pdr.cpp: einen ProxReport von "<<tagOid<<endl<<flush;
#endif
                    // hier Umwandlung aller relevanten Werte
                    for(int i=0; i<PROX_MAX; i++) {
                        PTNP::LocatableProximityTag* tag2; // das kontaktierte Tag
                        PTNP::OuttimingTagContact* contact; // der Kontakt zwischen den beiden Tags
                        PTNP::oid tag2Oid; // die OID des kontaktierten Tags
                        PTNP::signalStrength strength; // die Signalstärke zwischen den Tags

                        tbeacon->pkt.p.prox.oid_prox[i] = ntohs(tbeacon->pkt.p.prox.oid_prox[i]);
                        tag2Oid = tbeacon->pkt.p.prox.oid_prox[i] & 0x7FF; // die OID des kontaktierten Tags steht in den ersten 11 Bit
                        strength = ((tbeacon->pkt.p.prox.oid_prox[i]>>14)&0x3); //*0x40; // die Signalstärke in den letzten 2 Bit

                        if(tag2Oid!=0) {
#ifdef _DEV_
                        dec(cout)<<"pdr.cpp: Kontakt: "<<tagOid<<" mit "<<tag2Oid<<"; Signalstaerke: "<<strength<<endl<<flush;
#endif
                            tag2 = tagReference(tag2Oid);

                            PTNP::TagContactKey key(tagOid, tag2Oid);
                            if (_model.contacts().contains(key)) {
                                contact = _model.contacts().value(key);
                            } else {
                                contact = new PTNP::OuttimingTagContact(tag, tag2, antenna, strength, this);
                                connect(contact, SIGNAL(timeout()), this, SLOT(contactTimedOut()));
                                _model.contacts().insert(key,contact);
                                contact->start();
                            }
                            // hier die Behandlung für die fixierten Tags
                            if(tag->fixed()) {
                                tag2->seen(*tag, strength);
                            } else if (tag2->fixed()) {
                                tag->seen(*tag2, strength);
                            }
                            contact->seen(antenna, strength);
                            createTagToTagContactRunningPackage(tagOid, tag2Oid, strength, antennaID);
                        }
                    }
                    seq = (PTNP::sequenceNumber)tbeacon->pkt.p.prox.seq>>3;
                    tag->seen(antennaID,3);
                    createBeaconTrackerPackage(tagOid, 3, antennaID, seq);
                    break;
                case RFBPROTO_PROXTRACKER:
                    // das wird eigentlich nie gesendet ...
#ifdef _DEV_
                    cout<<"einen BeaconProxTracker von "<<tagOid<<"; - wird nicht behandelt";
#endif
                    break;
                default:
                    unknown = true;
                }


                if(tbeacon->pkt.flags & PTNP::BUTTON_PRESSED_FLAG) { // wenn der Button gedrückt wurde
                    PTNP::ButtonEvent* e;
#ifdef _DEV_
                    cout<<" Button!";
#endif
                    if(_model.buttons().contains(tagOid)) {
                        e = _model.buttons().value(tagOid);
                        e->stillPressed();
                    } else {
                        e = new PTNP::ButtonEvent(tag);
                        connect(e, SIGNAL(timeout()), this, SLOT(buttonTimedOut()));
                        _model.buttons().insert(tagOid, e);
                        e->start();
                    }
                }
            }
            if(unknown) {
#ifdef _DEV_
                cout<<"einen unbekannten Pakettyp "<<tbeacon->pkt.proto<<"; Paket wird verworfen!"<<flush;
#endif
            }
#ifdef _DEV_
            cout<<endl<<endl<<flush;
#endif
        } else {
#ifdef _DEV_
            QTextStream cerr(stderr, QIODevice::WriteOnly);
            cerr<<"pdr.cpp: crc-pruefsumme falsch! Paket wird verworfen!"<<endl<<endl<<flush;
#endif
        }
    } else {
#ifdef _DEV_
        cout<<"pdr.cpp: Paketgroesse: "<<datagram->size()<<"; erwarte: "<<sizeof(TBeaconPackage)<<"  Paket wird verworfen!"<<endl<<endl<<flush;
#endif
    }
    delete datagram;
}

/*!
  dieser Slot wird aufgerufen, sobald ein Kontakt ausläuft.
  Es wird ein Paket generiert und in die Warteschlange eingereiht, sowie die Kontaktdaten
  aus der Datenstruktur gelöscht.
  */
void ProximityDataReceiver::contactTimedOut() {
    PTNP::OuttimingTagContact* contact = qobject_cast<PTNP::OuttimingTagContact*>(sender());
    PTNP::flags f = 0;
    if(contact->canBeIgnored())
        f |= PTNP::IGNORE_FLAG;
    this->createTagToTagContactNewsPackage(contact->tag1()->oid(), contact->tag2()->oid(), contact->timeElapsedInSeconds(), contact->lastView(), f);

#ifdef _DEV_
    QTextStream cout(stdout, QIODevice::WriteOnly);
    cout<<"Kontakt vorbei!"<<endl<<flush;
#endif
    PTNP::TagContactKey key(contact->tag1()->oid(), contact->tag2()->oid());
    _model.contacts().take(key)->deleteLater();
}

/*!
  Dieser Slot wird aufgerufen, wenn der Button eines Tags nicht mehr gedrückt wird.
  Es wird ein Paket für die Warteschlange generiert, in diese eingetragen und die Buttondaten werden gelöscht.
  */
void ProximityDataReceiver::buttonTimedOut() {
    PTNP::ProximityTag* tag;
    PTNP::oid tagOid;
    PTNP::ButtonEvent* e;
    PTNP::Package* paket;
    QByteArray* packageData;

    e = qobject_cast<PTNP::ButtonEvent*>(sender());
    tagOid = e->tag()->oid(); // holt die TagID, um
    tag = _model.tags().value(tagOid); // das Tag zu holen

    // Paket zusammensetzen
    packageData = new QByteArray();
    QDataStream packageDataStream(packageData, QIODevice::WriteOnly);
    packageDataStream<<tagOid;
    PTNP::duration duration = e->timeElapsedInSeconds();
    packageDataStream<<duration;
    paket = new PTNP::Package(PTNP::BUTTON_MSG, packageData);
    _packageQueueMutex.lock();
    _queue.append(paket);
    _packageQueueMutex.unlock();
    emit receivedData();

#ifdef _DEV_
    QTextStream cout(stdout, QIODevice::WriteOnly);
    cout<<"Button losgelassen nach "<< e->timeElapsedInSeconds()<<" Sekunden!"<<endl<<flush;
#endif
    _model.buttons().take(tagOid)->deleteLater();
}

/*!
  Dieser Slot wird aufgerufen, wenn das Tag austimet - wenn es also von der Karte verschwunden ist bzw. keine neuen Nachrichten mehr empfangen werden.
  Es löscht das Tag aus der Liste und fortan werden keine Nachrichten mehr versendet (beispielsweise Positionsnachrichten, die sonst auf der letzten Position verharren würden).
  Kontakte, die dieses Tag noch hatte werden gelöscht
  */
void ProximityDataReceiver::tagTimedOut() {
    PTNP::LocatableProximityTag* tag = qobject_cast<PTNP::LocatableProximityTag*>(sender());
    if(!_model.contacts().empty()) {
        QList<PTNP::TagContactKey> contactKeys  = _model.contacts().uniqueKeys();
        PTNP::TagContactKey key = contactKeys.first();
        foreach(key, contactKeys) {
            if(key.contains(tag->oid())) { // Achtung: leider sehr aufwändig bei vielen Kontakten!!!
                PTNP::OuttimingTagContact* contact = _model.contacts().take(key);
                contact->abort(); // Kontakt abbrechen
                contact->deleteLater(); // kontakt aus der Liste entfernen
            }
        }
    }
#ifdef _DEV_
    QTextStream cout(stdout, QIODevice::WriteOnly);
    dec(cout)<<"tag"<<tag->oid()<<" zum loeschen gemerkt!"<<endl<<flush;
#endif
    _model.tags().take(tag->oid())->deleteLater();
}

/*!
  Berechnet die CRC16-Summe zum übergebenen buffer.
  Code vom OpenBeacon-Projekt übernommen.
  \param buffer die übergebenen Zeichen
  \param size die Anzahl der Zeichen im buffer
  */
quint16 ProximityDataReceiver::CRC16(unsigned char * buffer, int size) {
    quint16 crc=0xFFFF;
    if(buffer) {
        while(size--) {
            crc  =(crc>>8)|(crc<<8);
            crc ^=*buffer++;
            crc ^=((quint8)crc)>>4;
            crc ^=crc<<12;
            crc ^=(crc &0xFF) << 5;
        }
    }
    return crc;
}

/*!
  Dekodiert den Vektor v mit dem key tea_key.
  Code vom OpenBeacon-Projekt übernommen.
  \param v der zu dekodierende Vektor
  \param length die Länge des Vektors
  \param tea_key der Schlüssel nach dem entschlüsselt wird
  */
void ProximityDataReceiver::tea_block_decode(quint32 *v, quint32 length, const long *tea_key)
{
    u_int32_t z, y = v[0], sum=0,e,p,q;
    if(!v || !tea_key)
        return;
    q=6+52/length;
    sum=q*DELTA;
    while(sum) {
        e= (sum>>2) &3;
        for(p=length-1; p>0; p--)
            z=v[p-1],y=v[p]-=MX;
        z=v[length-1];
        y=v[0]-=MX;
        sum-=DELTA;
    }
}

/*!
  Diese Methode enwirrt das übergebene Paket.
  Dazu wird zunächst von Netzwerkbyteorder in Hostbyteorder umgewandelt
  (long), anschließend tea_block_decode aufgerufen und schließlich
  noch einmal von Netzwerkbyteorder in Hostebyteorder umgewandelt.
  Code vom OpenBeacon-Projekt übernommen.
  \param datagram Das zu entwirrende Paket
  */
void ProximityDataReceiver::decryptDatagram(QByteArray *datagram)
{
    int i;
    quint32* data = (quint32*) (datagram->data());
    for(i=0; i<XXTEA_BLOCK_COUNT; i++) {
        *data++=(ntohl(*data));
    }
    data = (quint32*) (datagram->data());
    tea_block_decode(data, XXTEA_BLOCK_COUNT,tea_key);
    for(i=0; i<XXTEA_BLOCK_COUNT; i++) {
        *data++=(ntohl(*data));
    }
}

/*!
  Diese Methode erstellt ein Beacontracker-paket und reiht es in die Warteschlange ein.
  Am Ende wird \sa receiveData emitiert.
  */
void ProximityDataReceiver::createBeaconTrackerPackage(PTNP::oid oid, PTNP::signalStrength strength, PTNP::aid aid, PTNP::sequenceNumber seq)
{
    QByteArray* packageData = new QByteArray();
    QDataStream packageDataStream(packageData, QIODevice::WriteOnly);

    packageDataStream<<oid;
    packageDataStream<<aid;
    packageDataStream<<strength;
    packageDataStream<<seq;
    _packageQueueMutex.lock();
    _queue.append(new PTNP::Package(PTNP::BEACON_MSG, packageData));
    _packageQueueMutex.unlock();
    emit receivedData();
}

/*!
  Dieser Slot erstellt ein TagPosition-paket und reiht es in die Warteschlange ein.
  Am Ende wird \sa receiveData emitiert.
  */
void ProximityDataReceiver::createTagPositionPackage(PTNP::oid oid1, PTNP::position pos) {
    QByteArray *packageData = new QByteArray();
    QDataStream packageDataStream(packageData, QIODevice::WriteOnly);

    packageDataStream<<oid1;
    packageDataStream<<pos.x;
    packageDataStream<<pos.y;
    packageDataStream<<pos.f;
    _packageQueueMutex.lock();
    _queue.append(new PTNP::Package(PTNP::POSITION_MSG, packageData));
    _packageQueueMutex.unlock();
    emit receivedData();
}

/*!
  Diese Methode erstellt ein TagToTagContactNews-paket und reiht es in die Warteschlange ein.
  Am Ende wird \sa receiveData emitiert.
  */
void ProximityDataReceiver::createTagToTagContactNewsPackage(PTNP::oid oid1, PTNP::oid oid2, PTNP::duration duration, PTNP::timestamp end, PTNP::flags flags) {
    QByteArray *packageData = new QByteArray();
    QDataStream packageDataStream(packageData, QIODevice::WriteOnly);

    packageDataStream<<oid1;
    packageDataStream<<oid2;
    packageDataStream<<duration;
    packageDataStream<<end;
    packageDataStream<<flags;
    _packageQueueMutex.lock();
    _queue.append(new PTNP::Package(PTNP::TAG_TO_TAG_CONTACT_NEWS, packageData));
    _packageQueueMutex.unlock();
    emit receivedData();
}

/*!
  Diese Methode erstellt ein TagToTagContactRunning-paket und reiht es in die Warteschlange ein.
  Am Ende wird \sa receiveData emitiert.
  */
void ProximityDataReceiver::createTagToTagContactRunningPackage(PTNP::oid oid1, PTNP::oid oid2, PTNP::signalStrength strength, PTNP::aid aid)
{
    QByteArray* packageData = new QByteArray();
    QDataStream packageDataStream(packageData, QIODevice::WriteOnly);

    packageDataStream<<oid1;
    packageDataStream<<oid2;
    packageDataStream<<aid;
    packageDataStream<<strength;
    _packageQueueMutex.lock();
    _queue.append(new PTNP::Package(PTNP::TAG_TO_TAG_CONTACT_RUNNING, packageData));
    _packageQueueMutex.unlock();
    emit receivedData();
}

/*!
  Gibt aus der Liste der Tags das mit der id tagOid zurück.
  Diese Methode fügt der Liste neue Tags zu, sobald eine unbekannte tagOid übergeben wird!
  Dabei wird auch geguckt, ob das Tag ein fixiertes Tag ist.
  */
PTNP::LocatableProximityTag* ProximityDataReceiver::tagReference(PTNP::oid tagOid) {
    PTNP::LocatableProximityTag* tag;
    // gucken, ob das Tag, dass die Nachricht gesendet hat überhaupt schon bekannt ist
    if(!(_model.tags().contains(tagOid))) {
        tag = new PTNP::LocatableProximityTag(tagOid, this);
        connect(tag, SIGNAL(timeout()), this, SLOT(tagTimedOut()));
        tag->start();
        _model.tags().insert(tagOid, tag);
    } else {
        tag = _model.tags().value(tagOid);
    }
    return tag;
}

/*!
  Gibt aus der Liste der Antennen die mit der id antennaID zurück.
  Diese Methode fügt der Liste neue Tags zu, sobald eine unbekannte antennaID übergeben wird!
  */
PTNP::ProximityAntenna* ProximityDataReceiver::antennaReference(PTNP::aid antennaID) {
    PTNP::ProximityAntenna* antenna;
    if(_model.antennas().contains(antennaID)) {
        antenna = _model.antennas().value(antennaID);
    } else {
#ifdef _DEV_
        QTextStream cout(stdout, QIODevice::WriteOnly);
        cout<<"pdr.cpp: new Antenna found! "<<antennaID<<endl<<flush;
#endif
        antenna = new PTNP::ProximityAntenna(antennaID);
        _model.antennas().insert(antennaID, antenna);
        emit newAntennaFound();
    }
    return antenna;
}

