#ifndef PROXIMITYRECEIVER_H
#define PROXIMITYRECEIVER_H

//#include <QThread>
#include <QHash>
#include <QQueue>
#include <QMutex>
#include <QFile>

#include "proximitylogger.h"
#include "proximitylocator.h"
#include "../common/protocol/PTNP.h"
#include "../common/protocol/package.h"
#include "../common/protocol/openbeacon.h"
#include "../common/protocol/tagcontactkey.h"
#include "../common/protocol/outtimingtagcontact.h"
#include "../common/protocol/beaconkey.h"
#include "../common/protocol/proximityantenna.h"
#include "../common/protocol/buttonevent.h"
#include "../common/protocol/locatableproximitytag.h"
#include "proximitydatamodel.h"

//! Das ist der Paketempfänger
/*!
  ein Objekt dieser Klasse nimmt über UDP ProximityPakete entgegen und legt sie nach Vorverarbeitung in der serverseitigen Warteschlange ab
  */
class ProximityDataReceiver : public QObject
{
    Q_OBJECT
public:
    //! Dieser Konstruktor kreiert einen Empfänger mit entsprechender Warteschlange
    ProximityDataReceiver(QQueue<PTNP::Package*> &queue, QMutex &packageQueueMutex, ProximityLogger *logger, ProximityDataModel &model);
    //! löscht den Socket
    ~ProximityDataReceiver();

signals:
    //! wird ausgelöst, wenn Daten am socket empfangen werden
    void receivedData();
    //! wird ausgelöst, wenn eine neue Antenne gefunden wurde
    void newAntennaFound();

public slots:
    //! hier werden entgegengenommene Pakete verarbeitet und als Pakete in die Warteschlange eingetragen
    void processDatagram(QByteArray* datagram, bool decrypt);
    void createTagPositionPackage(PTNP::oid oid1, PTNP::position pos);

private slots:
    //! wenn ein Kontakt austimt
    void contactTimedOut();
    //! wenn ein Button nicht mehr gedrückt wird
    void buttonTimedOut();
    //! wenn ein Tag verschwindet
    void tagTimedOut();

private:
    //! berechnet die CRC16-Summe für das empfangene Paket
    quint16 CRC16(unsigned char* buffer, int size);
    //! dekodiert ein empfangenes Paket
    void tea_block_decode(quint32 *v, quint32 length, const long *tea_key);
    //! entschlüsselt empfangene Pakete
    void decryptDatagram(QByteArray* datagram);

    void createBeaconTrackerPackage(PTNP::oid oid, PTNP::signalStrength strength, PTNP::aid aid, PTNP::sequenceNumber seq);
    void createTagToTagContactNewsPackage(PTNP::oid oid1, PTNP::oid oid2, PTNP::duration duration, PTNP::timestamp end, PTNP::flags flags);
    void createTagToTagContactRunningPackage(PTNP::oid oid1, PTNP::oid oid2, PTNP::signalStrength strength, PTNP::aid aid);

    PTNP::LocatableProximityTag* tagReference(PTNP::oid);
    PTNP::ProximityAntenna* antennaReference(PTNP::aid);

    QQueue<PTNP::Package*> &_queue;
    QMutex &_packageQueueMutex;
    ProximityLogger *_logger;
    bool _log;
    ProximityDataModel& _model;
};

#endif // PROXIMITYRECEIVER_H
