#ifndef PROXIMITYDISTRIBUTOR_H
#define PROXIMITYDISTRIBUTOR_H

#include <QMutex>
#include <QQueue>
#include "proximityserverclient.h"
#include "proximitydatamodel.h"
#include "../common/protocol/package.h"
#include "../common/protocol/PTNP.h"

//! Das ist der Paketverteiler
/*!
  Objekte dieser Klasse kümmern sich um die Verteilung der Warteschlangenpakete auf dem Server an eine Liste von Clients.
  Es werden den entsprechenden Clients nur die Pakete gesandt, die sie auch haben wollen.
  */
class ProximityDistributor : public QObject
{
    Q_OBJECT
public:
    //! Dieser Konstruktor kreiert den Verteiler
    ProximityDistributor(QList<ProximityServerClient*> &clients,
                         QMutex &clientListMutex,
                         QQueue<PTNP::Package*> &packageQueue,
                         QMutex &packageQueueMutex,
                         ProximityDataModel &model);
    //! tut momentan nichts
    ~ProximityDistributor();

public slots:
    //! hier passiert die eigetlichte Verteilung der Daten an die Clients
    void distribute();
    void distributeAllAntennasAndFixedTags();
private:
    void send(PTNP::Package* paket);
    QList<ProximityServerClient*> &_clients;
    QMutex &_clientListMutex;
    QQueue<PTNP::Package*>  &_packageQueue;
    QMutex &_packageQueueMutex;
    ProximityDataModel& _model;
};

#endif // PROXIMITYDISTRIBUTOR_H
