#include "proximitylocator.h"
#include <QSettings>
#include <QTextStream>

#include "math.h"

ProximityLocator::ProximityLocator(ProximityDataModel& model,
                                   int timeout)
    : _model(model)
{
    QSettings settings;
    _tagweight = settings.value("defaulttagweight","5.0").toReal();
#ifdef _DEV_
    QTextStream cout(stdout, QIODevice::WriteOnly);
    cout<<"pl.cpp: defaulttagweight: "<<_tagweight<<endl;
#endif
    quint8 numSignalStrength = settings.beginReadArray("signalstrengthweights");
    _signalStrengthWeight = QVector<quint8>(numSignalStrength);

#ifdef _DEV_
    cout<<"pl.cpp: reading "<<numSignalStrength<<" signalstrengthweights"<<endl;
#endif
    for(quint8 i=0; i<numSignalStrength; ++i) {
        settings.setArrayIndex(i);
        _signalStrengthWeight[i] = settings.value("weight").toUInt();
#ifdef _DEV_
        cout<<"pl.cpp: weight "<<i<<" "<<_signalStrengthWeight[i]<<endl;
#endif
    }
    _timer = new QTimer(this);
    _timer->setInterval(timeout);
    connect(_timer, SIGNAL(timeout()), this, SLOT(calculate()));
    _timer->start();
}

ProximityLocator::~ProximityLocator()
{
    _timer->stop();
    delete _timer;
}

QPoint ProximityLocator::getMovement(PTNP::LocatableProximityTag &tag) const {
    QMap<PTNP::aid, quint16> attractions = tag.getAntennaAttractions();
    QPoint entireMovement;
    QPoint tagPos(tag.pos().x, tag.pos().y);
    foreach(PTNP::aid id, attractions.uniqueKeys()) {
        PTNP::ProximityAntenna* a = _model.antennas().value(id);
        QPoint antennaPos(a->pos().x, a->pos().y);
        QPoint delta = antennaPos - tagPos;
        delta *= ((attractions.value(id)/_tagweight)/100);
        entireMovement += delta;
    }
#ifdef _DEV_
    QTextStream cout(stdout, QIODevice::WriteOnly);
    cout<<"pl.cpp "<<tag.oid()<<" entireMovement: "<<(sqrt(pow(entireMovement.x(), 2)+ pow(entireMovement.y(), 2)))<<endl;
#endif
    return entireMovement;
}

