#ifndef PROXIMITYLOCATOR_H
#define PROXIMITYLOCATOR_H

#include <QObject>
#include <QTimer>
#include <QPoint>
#include "../common/protocol/PTNP.h"
#include "proximitydatamodel.h"

#define LOCATOR_TIMEOUT 500

/*!
  Diese abstrakte Klasse bildet die Schnittstelle der Applikation zu den
  verscheidenen Ortungsalgorithmen.
  Jeder abgeleitete Algorithmus implementiert calculate als Slot, der per
  Timer immer wieder aufgerufen wird.
  */
class ProximityLocator : public QObject
{
    Q_OBJECT
public:
    //! Dieser Konstruktor erstellt einen abstrakten Locator für die Proximitytags und initialisiert den Timer mit dem Timeout timeout ms.
    ProximityLocator(ProximityDataModel& model,
                     int timeout = LOCATOR_TIMEOUT);
    //! Der Destruktor stopt den Timer und löscht alle Elemente, die von diesem Objekt auf dem Heap erstellt wurden
    ~ProximityLocator();
public slots:
    //! Diese Methode wird von allen Ortungsalgorithmen implementiert
    virtual void calculate() = 0;
signals:
    void sendPositionMessage(PTNP::oid, PTNP::position);
protected:
    ProximityDataModel& _model;
    QTimer* _timer;
    QVector<quint8> _signalStrengthWeight;
    qreal _tagweight;
    QPoint getMovement(PTNP::LocatableProximityTag& tag) const;
};

#endif // PROXIMITYLOCATOR_H
