#ifndef PROXIMITYSERVER_H
#define PROXIMITYSERVER_H

#include <QtCore/QObject>
#include <QTcpServer>
#include <QQueue>
#include <QMutex>
#include <QList>
#include "../common/protocol/PTNP.h"
#include "../common/protocol/package.h"
#include "proximityserverclient.h"
#include "proximitydatareceiver.h"
#include "proximitydistributor.h"
#include "proximityplayer.h"
#include "proximitylogger.h"
#include "proximitydataprovider.h"
#include "proximitylocator.h"
#include "proximitydatamodel.h"

#define PORT 3333

#define LOGFILE_PLAYER 0
#define NETWORK_PLAYER 1
#define SERIAL_PLAYER 2
#define LOGFILE_PLAYER_LOOP 3

//! Der ProximityServer
/*!
  Dieser Server lauscht auf Events von den Antennen und bietet diese vorverarbeitet verschiedenen Clients an.
  Die Clients können vorverarbeitete Events vom Server entgegennehmen.
  Des weiteren können Sequenzen von empfangenen Paketen aufgenommen und abgespielt werden.
  */
class ProximityServer : public QObject
{
    Q_OBJECT
public:
    //! der Standardkonstruktor für den Server
    ProximityServer(quint8 modus=LOGFILE_PLAYER, bool log=false, QString sourcePath="");
    ~ProximityServer();

    //! startet den Server
    bool start();

private slots:
    //! initialisiert die Verbindung zu einem Client
    void initConnection();
    //! schließt die Verbindung zum Client bzw. räumt auf
    void closeConnection();

private:
    void initLocator(); /*!< Diese Methode initialisiert den Locator */
    ProximityDataReceiver* _receiver; /*!< der Receiver, der die Warteschlange füttert */
    ProximityDistributor* _distributor; /*!< der Distributor, der die Warteschlange abarbeitet */
    QQueue<PTNP::Package*>* _packageQueue; /*!< eine Paketwarteschlange */
    QMutex* _packageQueueMutex; /*!< Zugriffsmutex für die Paketwarteschlange */
    QList<ProximityServerClient*>* _clients; /*!< Liste der offenen ClientSockets */
    QMutex* _clientListMutex; /*!< ZugriffsMutex für die ClientListe */
    QTcpServer* _server; /*!< der TcpServer, auf den der Server aufsetzt */
    ProximityPlayer* _player; /*!< der player spielt eine aufgenommen (geloggte) Datei ab */
    ProximityDataProvider* _dataProvider;
    ProximityLogger* _logger; /*!< der logger logt alle Daten, die der Receiver empfängt */
    ProximityLocator* _locator; /*!< der locator positioniert die Tags anhand der Antennenpositionen im Raum */
    quint8 _modus;
    bool _log; /*!< ob geloggt werden soll, oder nicht, entscheidet nicht das grüne Licht, sondern diese Variable! */
    QTextStream* cin;
    ProximityDataModel* _dataModel; /*!< das ist das Modell aus dem MVC*/
};

#endif // PROXIMITYSERVER_H
