#include "referencecontactlocator.h"

ReferenceContactLocator::ReferenceContactLocator(ProximityDataModel& model,
                                                 int timeout)
    : ProximityLocator(model, timeout)
{
}

ReferenceContactLocator::~ReferenceContactLocator()
{
}

QPoint ReferenceContactLocator::getMovement(PTNP::LocatableProximityTag &tag) const {
    QMap<PTNP::aid, quint16> antennaAttractions = tag.getAntennaAttractions();
    QMap<PTNP::oid, quint16> referenceAttractions = tag.getFixedTagAttractions();
    QPoint entireMovement;
    QPoint tagPos(tag.pos().x, tag.pos().y);
    foreach(PTNP::aid id, antennaAttractions.uniqueKeys()) {
        PTNP::ProximityAntenna* a = _model.antennas().value(id);
        QPoint antennaPos(a->pos().x, a->pos().y);
        QPoint delta = antennaPos - tagPos;
        delta *= ((antennaAttractions.value(id) / _tagweight) / 100);
        entireMovement += delta;
    }
    foreach(PTNP::oid id, referenceAttractions.uniqueKeys()) {
        PTNP::LocatableProximityTag* t = _model.fixedTags().value(id);
        QPoint referencePos(t->pos().x, t->pos().y);
        QPoint delta = referencePos - tagPos;
        delta *= ((referenceAttractions.value(id) / _tagweight) / 100);
        entireMovement += delta;
    }
    return entireMovement;
}

void ReferenceContactLocator::calculate()
{
    foreach(PTNP::LocatableProximityTag* tag, _model.tags()) {
        if(!tag->fixed()) {
            QPoint entireMovement = getMovement(*tag);
            QPoint tagPos (tag->pos().x, tag->pos().y);
            tagPos += entireMovement;
            PTNP::position newPos;
            newPos.f = tag->pos().f;
            newPos.x = tagPos.x();
            newPos.y = tagPos.y();
            tag->setPosition(newPos);
            emit sendPositionMessage(tag->oid(), newPos);
        }
    }
}
