#ifndef PTNP_H
#define PTNP_H

// benötigt für Datentypen quint8 etc.
#include <QObject>

/*!
  Hier wird alles Nötige für das Protokoll definiert
  */
namespace PTNP {

    //! neuer Typ oid für die IDs der Tags
    typedef quint16 oid;
    //! neuer Typ aid für die IDs der Antennen
    typedef quint16 aid;
    //! neuer Typ für Dauer
    typedef quint16 duration;
    //! neuer Typ signalStrength für die Signalstärke
    typedef quint8 signalStrength;
    //! neuer Typ für Zeitstempel
    typedef quint32 timestamp;
    //! neuer Typ für Sequenznummer
    typedef quint32 sequenceNumber;
    //! neuer Typ für die Nachrichtentypen
    typedef quint8 msgType;
    //! neuer Typ für Längen - Angabe in ganzen cm
    typedef quint16 coord;
    //! neuer Typ für Etage
    typedef quint8 floor;
    //! neuer Typ für Flags
    typedef quint8 flags;

    //! Diese Werte sind für die Ortung wichtig
    static const quint8 signalStrengthWeight[] = {100,64,9,1};

    // das Flag, dass gesetzt ist, wenn der Button gedrückt ist
    static const quint8 BUTTON_PRESSED_FLAG  = 0x02;

    //! Alle Antennen und fixierten Tags
    // schickt Anzahl der Antennen und fixierten Tags,
    // dann alle AntenneIDs inklusive flags
    // und hinzu alle fixierten Tags
    static const quint8 ALL_ANTENNAS_FIXED_TAGS     = 0x01;
    //! alle Informationen über abgeschlossene Kontakte zwischen Tags
    // schickt beide TagIDs und Dauer des Kontakts und den KontaktEndZeitstempel
    static const quint8 TAG_TO_TAG_CONTACT_NEWS     = 0x02;
    //! alle Informationen über laufende Kontakte zwischen Tags
    // schickt immer Pakete mit beiden TagIDs; Signalstärke zwischen ihnen und der AntennenID
    static const quint8 TAG_TO_TAG_CONTACT_RUNNING  = 0x04;
    //! alle Informationen zu den Tags (ohne Kontakte)
    // schickt TagID, AntennenID und Signalstärke
    static const quint8 BEACON_MSG                  = 0x08;
    //! alle Button-gedrückt-Informationen
    // schickt TagID und Dauer des Buttons der gedrückt wurde
    static const quint8 BUTTON_MSG                  = 0x10;
    // schickt Anzahl der Tags, deren Position sich änderte und dann alle Tags mit ID PosX und PosY
    static const quint8 POSITION_MSG                = 0x20;
    // noch möglich:
    // 0x40;
    // 0x80;
    //! alle verfügbaren Informationen
    // schickt alle oben genannten Pakete
    static const quint8 ALL                         = 0x3F;


    // Konstanten für die Flags
    //! Flag für Nachrichten aus Bereichen, wo beispielsweise keine Kontakte für die Datenbank getrackt werden sollen
    static const quint8 IGNORE_FLAG     = 0x01;

    // nun die entprechenden Structs für die Pakete des Protokolls
    typedef struct{
        coord x;
        coord y;
        floor f;
    } position;

    typedef struct{
        aid id;
        flags flag;
    } antenna;

    typedef struct{
        quint8 numAntennas;
        quint8 numFixedTags;
        antenna antennas[];
        oid fixedTags[];
    } allAntennas;

    typedef struct {
        oid tag1;
        oid tag2;
        duration dur;
        timestamp end;
        flags flag; // Achtung Änderung!!!!
    } tagToTagContactNews;

    typedef struct {
        oid tag1;
        oid tag2;
        aid ant;
        signalStrength strength;
    } tagToTagContactRunning;

    typedef struct {
        oid tag1;
        aid ant;
        signalStrength strength;
        sequenceNumber seq;
    } beaconMsg;

    typedef struct {
        oid tag1;
        duration dur;
    } buttonMsg;

    typedef struct {
        oid tag1;
        position pos;
    } tagPositionMsg;

    typedef union {
        allAntennas aa;
        tagToTagContactNews tttcn;
        tagToTagContactRunning tttcr;
        beaconMsg bem;
        buttonMsg bum;
        tagPositionMsg tpm;
    } payload;

    typedef struct {
        quint8 header; // der entscheidet, was es für ein struct im payload ist
        payload p;
    } package;

    // noch nicht verwendete Pakete
    typedef struct {
        quint8 header;
        quint32 passwd;
    } initPackage;

    typedef struct {
        quint8 header;
        quint8 response;
    } initResponse;
}

#endif // PTNP_H
