#ifndef PROXIMITYTAG_H
#define PROXIMITYTAG_H

#include "QDateTime"
#include "QMap"
#include "../protocol/openbeacon.h"
#include "PTNP.h"
#include "position.h"

namespace PTNP{
    /*!
      Diese Klasse beschreibt ein ProximityTag.
      Dieses besteht vor allem aus der ID des Tags und einem updatebaren Timestamp, in dem gespeichert wird, wann das Tag zuletzt gesehen wurde.
      */
    class ProximityTag : public TBeaconWrapper, public Position
            // muss für Clients schon von Position abgeleitet sein (die clients wollen keine "locatableproximitytags", aber dennoch Positionen)
    {
    public:
        //! Der Standardkostruktor erstellt ein ProximityTag mit ID und setzt den Zeitpunkt der letzten Sichtung auf die aktelle Zeit
        ProximityTag(PTNP::oid oid, bool fixed=false);
        //! lösche alle vom Objekt erstellten Daten
        ~ProximityTag();

        //! \returns Zeitpunkt der letzten Sichtung
        QDateTime lastSeen() const { return _lastSeen; }
        //! diese Methode aktualisiert den Zeitstempel auf die aktuelle Zeit
        void seen();
        //! gibt zurück, ob das Tag eine feste Position hat
        bool fixed() const { return _fixed;}
        //! setzt das Tag fixiert an die Position
        void setFixed(bool fixed) { _fixed = fixed; }
        //! gibt die oid zurück
        PTNP::oid oid() const {return TBeaconWrapper::oid;}
        //! gibt die Sequenznummer zurück
        quint32 seq() const { return TBeaconWrapper::p.tracker.seq; }
        //! setzt die Sequenznummer neu, wenn die neue größer ist, als die alte
        void setSeq(quint32 newSeq) { if(newSeq>TBeaconWrapper::p.tracker.seq) TBeaconWrapper::p.tracker.seq = newSeq;}
        //! setzt die Sequenznummer neu, wenn die neue größer ist, als die alte
        void setSeq(quint16 newSeq) { if(newSeq>TBeaconWrapper::p.tracker.seq) TBeaconWrapper::p.tracker.seq = newSeq;}
        //! gibt die OID des zuletzt gesehenen Tags zurück
        PTNP::oid lastSeenOID() const { return TBeaconWrapper::p.tracker.oid_last_seen; }
        //! gibt den Powerupcount des Tags zurück
        quint16 powerupCount() const { return TBeaconWrapper::p.tracker.powerup_count; }
        //! vergleicht zwei Tags anhand der OID
        bool operator==(ProximityTag const& pt2) const;

    protected:
        QDateTime _lastSeen;
        bool _fixed;
    };
}

#endif // PROXIMITYTAG_H
