#ifndef TAGCONTACT_H
#define TAGCONTACT_H

#include "proximitytag.h"
#include "proximityantenna.h"
#include "tagcontactinfo.h"

namespace PTNP {

#define IGNORETHRESHOLD 0,5

    /*!
      Diese Klasse speichert zwei Tags und die Kontaktinformationen dazu
      */
    class TagContact
    {
    public:
        //! Der Konstruktor erstellt aus zwei Tags, einer Antenne und der Signalstärke, mit der sich die Tags sehen ein Kontaktobjekt
        TagContact(ProximityTag* const tag1, ProximityTag* const tag2, ProximityAntenna* const antenna, quint8 strength);
        //! löscht alle vom Objekt erstellten Daten
        ~TagContact();

        //! gibt zurück, ob dieser Kontakt ignoriert werden darf
        bool canBeIgnored();
        //! frischt die Kontaktinformationen auf
        // TODO: als SLOT implementieren
        void seen(ProximityAntenna* const antenna, quint8 strength);
        //! liefert das ein Tag zurück
        const ProximityTag* tag1() {return tag1_;}
        //! liefert das andere Tag zurück
        const ProximityTag* tag2() {return tag2_;}
        //! gibt die Dauer des Kontaktes in Sekunden zurück - Achtung: kann bis zu MAX_CONTACT_TIMEOUT * MULTIPLIER ms verzägert sein!
        PTNP::duration timeElapsedInSeconds() { return _actualDuration; }
        //! gibt den Zeitpunkt der letzten Sichtung zurück
        PTNP::timestamp lastView() {return _timestamp_LastView; }
        //! gibt den Zeitpunkt der ersten Sichtung zurück
        PTNP::timestamp firstView() {return _timestamp_FirstView; }


    protected:
        ProximityTag* const tag1_;
        ProximityTag* const tag2_;
        QList<TagContactInfo*> _contactInfo;
        PTNP::duration _actualDuration;
        PTNP::timestamp _timestamp_LastView;
        PTNP::timestamp _timestamp_FirstView;
        quint16 _ignoreCounter; // wie oft wurde der Kontakt von Antennen gesehen, deren Kontakte ignoriert werden sollen
        quint16 _seenCounter; // wie oft der Kontakt gesehen wurde
    };
}
#endif // TAGCONTACT_H
