#ifndef PROXIMITYCLIENTNEW_H
#define PROXIMITYCLIENTNEW_H

#include <QObject>
#include <QMap>
#include <QTcpSocket>
#include "protocol/PTNP.h"

#define SERVER_HOST 0
#define SERVER_PORT 1

class ProximityClient : public QObject
{
    Q_OBJECT
public:
    //! Konstruktor für einen Server mit Adresse, port und Nachrichtentyp (standard: alle)
    ProximityClient(QString server, quint16 port, PTNP::msgType msgType = PTNP::ALL);
    ~ProximityClient();
    //! gibt den aktuell zu empfangenden Nachrichtentypen zurück
    PTNP::msgType msgType() { return _msgType; }
    //! setzt den Server auf server
    void setServer(QString server) { this->_server = server; }
    //! setzt den Port auf port
    void setPort(quint16 port) { this->_port = port; }
    //! stellt eine neue Verbindung zum Server her
    void reconnect();
signals:
    // hier alle mäglichen Pakete, die empfangen worden sein können
    void tagToTagContactNews(PTNP::tagToTagContactNews);
    void tagToTagContactRunning(PTNP::tagToTagContactRunning);
    void beaconMsg(PTNP::beaconMsg);
    void buttonMsg(PTNP::buttonMsg);
    void allAntennas(QList<PTNP::antenna>, QList<PTNP::oid>);
    void tagPositionMsg(PTNP::tagPositionMsg);

    //! wird emitiert, wenn ein InitResponse-Paket ankam
    void initResponse(PTNP::initResponse);
    //! wird emitiert, wenn ein Socketerror auftrat
    void socketError(QAbstractSocket::SocketError);

private slots:
    void prepareSocket();
    void initConnection();
    void handleInitResponse(); // momentan ungenutzt
    void handleResponse();
    //! diese Methode fängt alle Socketfehler ab
    void handleSocketError(QAbstractSocket::SocketError e);
private:
    // Diese Methoden nehmen die einzelnen empfangenen Bytes auseinander
    // und sortieren sie in die entsprechenden Pakete ein
    void handleAllAntennasPackage();
    void handleTagPositionMsg();
    void handleBeaconPackage();
    void handleButtonPackage();
    void handleTagToTagContactNewsPackage();
    void handleTagToTagContactRunningPackage();

    PTNP::msgType _msgType;
    QString _server;
    quint16 _port;
    QTcpSocket *_socket;
};

#endif // PROXIMITYCLIENTNEW_H
